/************************************************************
 **
 ** cmd.h : p-code structure handling declarations
 **
 ** Copyright (c) 2004-2005, Kyle A. York
 ** All rights reserved
 **
 ************************************************************/
#ifndef cmd_h__
#define cmd_h__

#include "../libutils/cache.h"
#include "label.h"
#include "operator.h"
#include "value.h"

#define CMD_NONE ((cmd_t) 0)

typedef enum cmd_type_ {
  cmd_type_nop,           /* no operation             */
  cmd_type_label,         /* this holds a label       */
  cmd_type_operator,      /* operator                 */
  cmd_type_branch,        /* control                  */
  cmd_type_asm,           /* inline assembly          */
  cmd_type_end,           /* end of program           */
  cmd_type_sleep,         /* sleep                    */
  cmd_type_isr_cleanup,
  cmd_type_proc_enter,    /* enter a procedure        */
  cmd_type_proc_leave,    /* exit a procedure         */
  cmd_type_block_start,   /* enter a block            */
  cmd_type_block_end,     /* leave a block            */
  cmd_type_statement_end, /* end-of-statement         */
  cmd_type_usec_delay     /* delay (in usec)          */
} cmd_type_t;

typedef void (*cmd_dump_t)(cmd_t cmd, FILE *dst);
typedef void (*cmd_free_t)(cmd_t cmd);

typedef struct {
  cmd_free_t free_fn;
  cmd_dump_t dump_fn;
} cmd_vtbl_t;

#define CMD_FLAG_INTERRUPT 0x0001 /* used at interrupt level */
#define CMD_FLAG_USER      0x0002 /* used at user level      */


cmd_t           cmd_alloc(cmd_type_t type, const cmd_vtbl_t *vtbl);
void            cmd_free(cmd_t cmd);
void            cmd_list_free(cmd_t cmd);

cmd_t           cmd_proc_alloc(cmd_type_t type, pfile_proc_t *proc);
cmd_t           cmd_label_alloc(label_t lbl);
cmd_t           cmd_op_alloc(operator_t op, value_t dst, value_t val1,
                    value_t val2);

void            cmd_flag_set(cmd_t cmd, flag_t flag);
boolean_t       cmd_flag_test(const cmd_t cmd, flag_t flag);
void            cmd_flag_clr(cmd_t cmd, flag_t flag);
void            cmd_flag_set_all(cmd_t cmd, flag_t flags);


/* common operations */
void            cmd_dump(cmd_t cmd, FILE *dst);
void            cmd_free(cmd_t cmd);

cmd_t           cmd_link_get(const cmd_t cmd);
void            cmd_link_set(cmd_t cmd, cmd_t lnk);

cmd_type_t      cmd_type_get(const cmd_t cmd);
void            cmd_type_set(cmd_t cmd, cmd_type_t type);

unsigned        cmd_line_get(const cmd_t cmd);
void            cmd_line_set(cmd_t cmd, unsigned line);

pfile_source_t *cmd_source_get(const cmd_t cmd);
void            cmd_source_set(cmd_t cmd, pfile_source_t *src);

pfile_proc_t   *cmd_proc_get(const cmd_t cmd);
void            cmd_proc_set(cmd_t cmd, pfile_proc_t *proc);

label_t         cmd_label_get(const cmd_t cmd);
void            cmd_label_set(cmd_t cmd, label_t lbl);

boolean_t       cmd_is_executable(cmd_t cmd);
cmd_t           cmd_next_exec_get(cmd_t cmd);
void            cmd_remove(cmd_t *cmd_head, cmd_t cmd);
cmd_t           cmd_label_find(cmd_t cmd, const label_t lbl);

boolean_t       cmd_is_reachable(cmd_t cmd);

/* delay hack */
variable_const_t cmd_usec_delay_get(cmd_t cmd);
void             cmd_usec_delay_set(cmd_t cmd, variable_const_t n);

cmd_t            cmd_usec_delay_alloc(variable_const_t n);

#endif /* cmd_h__ */

